@@ -5,18 +5,17 @@ use std::path::{Path, PathBuf};
 use std::process::{Command, ExitStatus, Stdio};
 use std::sync::Arc;
 
-use config::ConfigError;
 use futures::StreamExt;
 use itertools::Itertools;
 use jj_lib::backend::{FileId, MergedTreeId, TreeValue};
 use jj_lib::conflicts::{self, materialize_merge_result};
+use jj_lib::fsmonitor::FsmonitorKind;
 use jj_lib::gitignore::GitIgnoreFile;
 use jj_lib::local_working_copy::{TreeState, TreeStateError};
 use jj_lib::matchers::Matcher;
 use jj_lib::merge::{Merge, MergedTreeValue};
 use jj_lib::merged_tree::{MergedTree, MergedTreeBuilder};
 use jj_lib::repo_path::{RepoPath, RepoPathBuf};
-use jj_lib::settings::UserSettings;
 use jj_lib::store::Store;
 use jj_lib::working_copy::{CheckoutError, SnapshotOptions};
 use pollster::FutureExt;
@@ -107,18 +106,11 @@ impl ExternalMergeTool {
 
 #[derive(Debug, Error)]
 pub enum ExternalToolError {
-    #[error("Invalid config")]
-    Config(#[from] ConfigError),
-    #[error(
-        "To use `{tool_name}` as a merge tool, the config `merge-tools.{tool_name}.merge-args` \
-         must be defined (see docs for details)"
-    )]
-    MergeArgsNotConfigured { tool_name: String },
     #[error("Error setting up temporary directory")]
     SetUpDir(#[source] std::io::Error),
-    // TODO: Remove the "(run with --verbose to see the exact invocation)"
-    // from this and other errors. Print it as a hint but only if --verbose is *not* set.
-    #[error("Error executing '{tool_binary}' (run with --verbose to see the exact invocation)")]
+    // TODO: Remove the "(run with --debug to see the exact invocation)"
+    // from this and other errors. Print it as a hint but only if --debug is *not* set.
+    #[error("Error executing '{tool_binary}' (run with --debug to see the exact invocation)")]
     FailedToExecute {
         tool_binary: String,
         #[source]
@@ -422,13 +414,12 @@ fn find_all_variables(args: &[String]) -> impl Iterator<Item = &str> {
 }
 
 pub fn edit_diff_external(
-    editor: ExternalMergeTool,
+    editor: &ExternalMergeTool,
     left_tree: &MergedTree,
     right_tree: &MergedTree,
     matcher: &dyn Matcher,
-    instructions: &str,
+    instructions: Option<&str>,
     base_ignores: Arc<GitIgnoreFile>,
-    settings: &UserSettings,
 ) -> Result<MergedTreeId, DiffEditError> {
     let got_output_field = find_all_variables(&editor.edit_args).contains(&"output");
     let store = left_tree.store();
@@ -451,10 +442,12 @@ pub fn edit_diff_external(
     let instructions_path_to_cleanup = output_wc_path.join("JJ-INSTRUCTIONS");
     // In the unlikely event that the file already exists, then the user will simply
     // not get any instructions.
-    let add_instructions = settings.diff_instructions()
-        && !instructions.is_empty()
-        && !instructions_path_to_cleanup.exists();
-    if add_instructions {
+    let add_instructions = if !instructions_path_to_cleanup.exists() {
+        instructions
+    } else {
+        None
+    };
+    if let Some(instructions) = add_instructions {
         let mut output_instructions_file =
             File::create(&instructions_path_to_cleanup).map_err(ExternalToolError::SetUpDir)?;
         if diff_wc.right_working_copy_path() != output_wc_path {
@@ -512,18 +505,19 @@ diff editing in mind and be a little inaccurate.
             exit_status,
         }));
     }
-    if add_instructions {
+    if add_instructions.is_some() {
         std::fs::remove_file(instructions_path_to_cleanup).ok();
     }
 
+    // Snapshot changes in the temporary output directory.
     let mut output_tree_state = diff_wc
         .output_tree_state
         .unwrap_or(diff_wc.right_tree_state);
     output_tree_state.snapshot(SnapshotOptions {
         base_ignores,
-        fsmonitor_kind: settings.fsmonitor_kind()?,
+        fsmonitor_kind: FsmonitorKind::None,
         progress: None,
-        max_new_file_size: settings.max_new_file_size()?,
+        max_new_file_size: u64::MAX,
     })?;
     Ok(output_tree_state.current_tree_id().clone())
 }
@@ -558,7 +552,7 @@ pub fn generate_diff(
             tool_binary: tool.program.clone(),
             source,
         })?;
-    io::copy(&mut child.stdout.take().unwrap(), writer).map_err(ExternalToolError::Io)?;
+    let copy_result = io::copy(&mut child.stdout.take().unwrap(), writer);
     // Non-zero exit code isn't an error. For example, the traditional diff command
     // will exit with 1 if inputs are different.
     let exit_status = child.wait().map_err(ExternalToolError::Io)?;
@@ -566,6 +560,7 @@ pub fn generate_diff(
     if !exit_status.success() {
         writeln!(ui.warning(), "{}", format_tool_aborted(&exit_status)).ok();
     }
+    copy_result.map_err(ExternalToolError::Io)?;
     Ok(())
 }
 
@@ -575,7 +570,7 @@ fn format_tool_aborted(exit_status: &ExitStatus) -> String {
         .map(|c| c.to_string())
         .unwrap_or_else(|| "<unknown>".to_string());
     format!(
-        "Tool exited with a non-zero code (run with --verbose to see the exact invocation). Exit \
+        "Tool exited with a non-zero code (run with --debug to see the exact invocation). Exit \
          code: {code}."
     )
 }
