@@ -146,15 +146,15 @@ fn test_rebase_descendants_forward() {
         .rebase_descendants_return_map(&settings)
         .unwrap();
     let new_commit_d =
-        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_d, &[&commit_f.id()]);
+        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_d, &[(commit_f.id())]);
     let new_commit_f =
-        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_f, &[&new_commit_d.id()]);
+        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_f, &[new_commit_d.id()]);
     let new_commit_c =
-        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_c, &[&new_commit_f.id()]);
+        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_c, &[new_commit_f.id()]);
     let new_commit_e =
-        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_e, &[&new_commit_d.id()]);
+        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_e, &[new_commit_d.id()]);
     let new_commit_g =
-        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_g, &[&new_commit_f.id()]);
+        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_g, &[new_commit_f.id()]);
     assert_eq!(rebase_map.len(), 5);
 
     assert_eq!(
@@ -206,8 +206,7 @@ fn test_rebase_descendants_reorder() {
         .mut_repo()
         .rebase_descendants_return_map(&settings)
         .unwrap();
-    let new_commit_i =
-        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_i, &[&commit_h.id()]);
+    let new_commit_i = assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_i, &[commit_h.id()]);
     assert_eq!(rebase_map.len(), 1);
 
     assert_eq!(
@@ -243,8 +242,7 @@ fn test_rebase_descendants_backward() {
         .mut_repo()
         .rebase_descendants_return_map(&settings)
         .unwrap();
-    let new_commit_d =
-        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_d, &[&commit_b.id()]);
+    let new_commit_d = assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_d, &[commit_b.id()]);
     assert_eq!(rebase_map.len(), 1);
 
     assert_eq!(
@@ -286,10 +284,9 @@ fn test_rebase_descendants_chain_becomes_branchy() {
         .mut_repo()
         .rebase_descendants_return_map(&settings)
         .unwrap();
-    let new_commit_f =
-        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_f, &[&commit_e.id()]);
+    let new_commit_f = assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_f, &[commit_e.id()]);
     let new_commit_d =
-        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_d, &[&new_commit_f.id()]);
+        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_d, &[new_commit_f.id()]);
     assert_eq!(rebase_map.len(), 2);
 
     assert_eq!(
@@ -331,15 +328,13 @@ fn test_rebase_descendants_internal_merge() {
         .mut_repo()
         .rebase_descendants_return_map(&settings)
         .unwrap();
-    let new_commit_c =
-        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_c, &[&commit_f.id()]);
-    let new_commit_d =
-        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_d, &[&commit_f.id()]);
+    let new_commit_c = assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_c, &[commit_f.id()]);
+    let new_commit_d = assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_d, &[commit_f.id()]);
     let new_commit_e = assert_rebased_onto(
         tx.mut_repo(),
         &rebase_map,
         &commit_e,
-        &[&new_commit_c.id(), &new_commit_d.id()],
+        &[new_commit_c.id(), new_commit_d.id()],
     );
     assert_eq!(rebase_map.len(), 3);
 
@@ -385,7 +380,7 @@ fn test_rebase_descendants_external_merge() {
         tx.mut_repo(),
         &rebase_map,
         &commit_e,
-        &[&commit_f.id(), &commit_d.id()],
+        &[commit_f.id(), commit_d.id()],
     );
     assert_eq!(rebase_map.len(), 1);
 
@@ -425,12 +420,10 @@ fn test_rebase_descendants_abandon() {
         .mut_repo()
         .rebase_descendants_return_map(&settings)
         .unwrap();
-    let new_commit_c =
-        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_c, &[&commit_a.id()]);
-    let new_commit_d =
-        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_d, &[&commit_a.id()]);
+    let new_commit_c = assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_c, &[commit_a.id()]);
+    let new_commit_d = assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_d, &[commit_a.id()]);
     let new_commit_f =
-        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_f, &[&new_commit_d.id()]);
+        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_f, &[new_commit_d.id()]);
     assert_eq!(rebase_map.len(), 3);
 
     assert_eq!(
@@ -504,8 +497,7 @@ fn test_rebase_descendants_abandon_and_replace() {
         .mut_repo()
         .rebase_descendants_return_map(&settings)
         .unwrap();
-    let new_commit_d =
-        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_d, &[&commit_e.id()]);
+    let new_commit_d = assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_d, &[commit_e.id()]);
     assert_eq!(rebase_map.len(), 1);
 
     assert_eq!(
@@ -514,9 +506,8 @@ fn test_rebase_descendants_abandon_and_replace() {
     );
 }
 
-// TODO(#2600): This behavior may need to change
 #[test]
-fn test_rebase_descendants_abandon_degenerate_merge() {
+fn test_rebase_descendants_abandon_degenerate_merge_simplify() {
     let settings = testutils::user_settings();
     let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
@@ -539,10 +530,61 @@ fn test_rebase_descendants_abandon_degenerate_merge() {
     tx.mut_repo().record_abandoned_commit(commit_b.id().clone());
     let rebase_map = tx
         .mut_repo()
-        .rebase_descendants_return_map(&settings)
+        .rebase_descendants_with_options_return_map(
+            &settings,
+            RebaseOptions {
+                simplify_ancestor_merge: true,
+                ..Default::default()
+            },
+        )
         .unwrap();
-    let new_commit_d =
-        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_d, &[&commit_c.id()]);
+    let new_commit_d = assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_d, &[commit_c.id()]);
+    assert_eq!(rebase_map.len(), 1);
+
+    assert_eq!(
+        *tx.mut_repo().view().heads(),
+        hashset! {new_commit_d.id().clone()}
+    );
+}
+
+#[test]
+fn test_rebase_descendants_abandon_degenerate_merge_preserve() {
+    let settings = testutils::user_settings();
+    let test_repo = TestRepo::init();
+    let repo = &test_repo.repo;
+
+    // Commit B was abandoned. Commit D should get rebased to have A and C as
+    // parents.
+    //
+    // D
+    // |\
+    // B C
+    // |/
+    // A
+    let mut tx = repo.start_transaction(&settings);
+    let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
+    let commit_a = graph_builder.initial_commit();
+    let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
+    let commit_c = graph_builder.commit_with_parents(&[&commit_a]);
+    let commit_d = graph_builder.commit_with_parents(&[&commit_b, &commit_c]);
+
+    tx.mut_repo().record_abandoned_commit(commit_b.id().clone());
+    let rebase_map = tx
+        .mut_repo()
+        .rebase_descendants_with_options_return_map(
+            &settings,
+            RebaseOptions {
+                simplify_ancestor_merge: false,
+                ..Default::default()
+            },
+        )
+        .unwrap();
+    let new_commit_d = assert_rebased_onto(
+        tx.mut_repo(),
+        &rebase_map,
+        &commit_d,
+        &[commit_a.id(), commit_c.id()],
+    );
     assert_eq!(rebase_map.len(), 1);
 
     assert_eq!(
@@ -585,7 +627,7 @@ fn test_rebase_descendants_abandon_widen_merge() {
         tx.mut_repo(),
         &rebase_map,
         &commit_f,
-        &[&commit_b.id(), &commit_c.id(), &commit_d.id()],
+        &[commit_b.id(), commit_c.id(), commit_d.id()],
     );
     assert_eq!(rebase_map.len(), 1);
 
@@ -626,10 +668,8 @@ fn test_rebase_descendants_multiple_sideways() {
         .mut_repo()
         .rebase_descendants_return_map(&settings)
         .unwrap();
-    let new_commit_c =
-        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_c, &[&commit_f.id()]);
-    let new_commit_e =
-        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_e, &[&commit_f.id()]);
+    let new_commit_c = assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_c, &[commit_f.id()]);
+    let new_commit_e = assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_e, &[commit_f.id()]);
     assert_eq!(rebase_map.len(), 2);
 
     assert_eq!(
@@ -764,7 +804,7 @@ fn test_rebase_descendants_divergent_rewrite() {
     let new_commit_c =
         assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_c, &[commit_b2.id()]);
     let new_commit_g =
-        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_g, &[&commit_f2.id()]);
+        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_g, &[commit_f2.id()]);
     assert_eq!(rebase_map.len(), 2); // Commit E is not rebased
 
     assert_eq!(
@@ -1543,6 +1583,7 @@ fn test_empty_commit_option(empty_behavior: EmptyBehaviour) {
             &settings,
             RebaseOptions {
                 empty: empty_behavior.clone(),
+                simplify_ancestor_merge: true,
             },
         )
         .unwrap();
@@ -1560,7 +1601,7 @@ fn test_empty_commit_option(empty_behavior: EmptyBehaviour) {
                 tx.mut_repo(),
                 &rebase_map,
                 &commit_f,
-                &[&new_commit_c.id(), &new_commit_d.id(), &new_commit_e.id()],
+                &[new_commit_c.id(), new_commit_d.id(), new_commit_e.id()],
             );
             let new_commit_g =
                 assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_g, &[new_commit_f.id()]);
@@ -1595,11 +1636,11 @@ fn test_empty_commit_option(empty_behavior: EmptyBehaviour) {
                 tx.mut_repo(),
                 &rebase_map,
                 &commit_f,
-                &[&new_commit_c.id(), &new_commit_e.id()],
+                &[new_commit_c.id(), new_commit_e.id()],
             );
             let new_commit_g =
-                assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_g, &[&new_commit_f.id()]);
-            assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_h, &[&new_commit_g.id()])
+                assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_g, &[new_commit_f.id()]);
+            assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_h, &[new_commit_g.id()])
         }
     };
 
@@ -1622,24 +1663,17 @@ fn test_rebase_abandoning_empty() {
     // Rebase B onto B2, where B2 and B have the same tree, abandoning all empty
     // commits.
     //
-    // We expect B, D, and G to be skipped because they're empty, but not E because
-    // it's the working commit. F also remains as it's not empty.
-    //
-    // F G (empty)                    F'
-    // |/                             |
-    // E (WC, empty)  D (empty)       E' (WC, empty)
-    // |             /                |
+    // We expect B, D, E, and G to be skipped because they're empty. F remains
+    // as it's not empty.
+    // F G (empty)
+    // |/
+    // E (WC, empty)  D (empty)       F' E' (WC, empty)
+    // |             /                |/
     // C-------------                 C'
     // |                           => |
     // B B2                           B2
     // |/                             |
     // A                              A
-    //
-    // TODO(#2869): There is a minor bug here. We'd like the result to be
-    // equivalent to rebasing and then `jj abandon`-ing all the empty commits.
-    // In case of the working copy, this would mean that F' would be a direct
-    // child of C', and the working copy would be a new commit that's also a
-    // direct child of C'.
 
     let mut tx = repo.start_transaction(&settings);
     let commit_a = write_random_commit(tx.mut_repo(), &settings);
@@ -1683,6 +1717,7 @@ fn test_rebase_abandoning_empty() {
 
     let rebase_options = RebaseOptions {
         empty: EmptyBehaviour::AbandonAllEmpty,
+        simplify_ancestor_merge: true,
     };
     rebase_commit_with_options(
         &settings,
@@ -1698,21 +1733,24 @@ fn test_rebase_abandoning_empty() {
         .unwrap();
     assert_eq!(rebase_map.len(), 5);
     let new_commit_c =
-        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_c, &[&commit_b2.id()]);
+        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_c, &[commit_b2.id()]);
     assert_abandoned_with_parent(tx.mut_repo(), &rebase_map, &commit_d, new_commit_c.id());
-    let new_commit_e =
-        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_e, &[&new_commit_c.id()]);
+    assert_abandoned_with_parent(tx.mut_repo(), &rebase_map, &commit_e, new_commit_c.id());
     let new_commit_f =
-        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_f, &[&new_commit_e.id()]);
-    assert_abandoned_with_parent(tx.mut_repo(), &rebase_map, &commit_g, new_commit_e.id());
+        assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit_f, &[new_commit_c.id()]);
+    assert_abandoned_with_parent(tx.mut_repo(), &rebase_map, &commit_g, new_commit_c.id());
+
+    let new_wc_commit_id = tx
+        .mut_repo()
+        .view()
+        .get_wc_commit_id(&workspace)
+        .unwrap()
+        .clone();
+    let new_wc_commit = tx.mut_repo().store().get_commit(&new_wc_commit_id).unwrap();
+    assert_eq!(new_wc_commit.parent_ids(), &[new_commit_c.id().clone()]);
 
     assert_eq!(
         *tx.mut_repo().view().heads(),
-        hashset! {new_commit_f.id().clone()}
-    );
-
-    assert_eq!(
-        tx.mut_repo().view().get_wc_commit_id(&workspace),
-        Some(new_commit_e.id())
+        hashset! {new_commit_f.id().clone(), new_wc_commit_id.clone()}
     );
 }
